require "fileutils"
require "securerandom"
require "nokogiri"

module Crosby
  class Exporter
    WRITE_MODE = "w"

    attr_reader :mail

    def initialize(mailer, method, uuid = SecureRandom.hex(8), *args)
      @args = args
      @mailer = mailer
      @method = method
      @uuid = uuid
      @version = Crosby.action_mailer_version

      config = Crosby.config
      @path = config[:export_path]
      @app_name = config[:app_name]
    end

    def export
      create_mail

      return false if ENV["SKIP_CROSBY"]

      ensure_path_exists
      write
      true
    end

    def to_s
      create_mail unless mail

      [
        "TO: #{mail.to}",
        "CC: #{mail.cc}",
        "BCC: #{mail.bcc}",
        "REPLY TO: #{mail.reply_to}",
        "FROM: #{mail.from}",
        "HTML:",
        commonize_html(mail.html_part.body.raw_source),
        "TEXT:",
        commonize_text(mail.text_part.body.raw_source)
      ].join("\n")
    end

    private

    def commonize_html(str)
      commonize_text(str)
      compact(Nokogiri::HTML::Document.parse(commonize_text(str)).to_s)
    end

    def commonize_text(str)
      str
        .gsub(/([^=])=\n/, '\1') # single quote required for encoding
        .gsub(/=3D/, "=")
    end

    def compact(str)
      str
        .each_line
        .reject(&:blank?)
        .map{ |line|
          line
            .gsub(/\s+/, " ")
            .gsub(/^ /, "")
            .gsub(/ $/, "\n")
            .gsub(/></, ">\n<")
        }
        .join
    end

    def create_mail
      method = @method

      case @version::MAJOR
      when 2
        method = "create_#{method}"
      when 3
        # supported, no additional action required
      when 4
        # supported, no additional action required
      else
        raise Crosby::UnsupportedVersion, "We do not yet support version " \
          "#{@version::STRING} of ActionMailer"
      end

      @mail = @mailer.send(method, *@args)
    end

    def ensure_path_exists
      @dir = Dir.new(FileUtils.mkdir_p(@path).first)
    end

    def file(&block)
      File.open(filename, WRITE_MODE, &block)
    end

    def filename
      File.join(@dir, "#{@mailer.name}.#{@method}.#{@app_name}.#{@uuid}.crosby")
    end

    def write
      file do |f|
        f.write(to_s)
      end
    end
  end
end
